@@ -2,12 +2,18 @@ from docutils import nodes
 from sphinx.ext.viewcode import collect_pages
 from sphinx.pycode import ModuleAnalyzer
 import imp
+import ast
 import re
-from sphinx.util.compat import Directive
+from docutils.parsers.rst import Directive
+import docutils
 import os
-from docutils.statemachine import StringList
-from sphinx.environment import NoUri
-from sphinx.pycode.pgen2 import token
+
+try:
+    # 3.0
+    from sphinx.errors import NoUri
+except:
+    from sphinx.environment import NoUri
+
 import warnings
 from . import util
 
@@ -20,9 +26,7 @@ else:
     text_type = str
 
 
-def view_source(
-    name, rawtext, text, lineno, inliner,
-        options={}, content=[]):
+def view_source(name, rawtext, text, lineno, inliner, options={}, content=[]):
 
     env = inliner.document.settings.env
 
@@ -31,25 +35,35 @@ def view_source(
 
 
 def _get_sphinx_py_module(env):
-    base_name = env.temp_data.get('autodoc:module', None)
+    base_name = env.temp_data.get("autodoc:module", None)
     if base_name is not None:
         return base_name
     if util.SPHINX_VERSION >= (1, 3):
-        base_name = env.ref_context.get('py:module', None)
+        base_name = env.ref_context.get("py:module", None)
         if base_name is not None:
             return base_name
     else:
-        base_name = env.temp_data.get('py:module', None)
+        base_name = env.temp_data.get("py:module", None)
         if base_name is not None:
             return base_name
 
     return None
 
 
+def _get_module_docstring(_file):
+    content = _file.read()
+    module = ast.parse(content)
+    for elem in module.body:
+        if isinstance(elem, ast.Expr) and isinstance(elem.value, ast.Str):
+            return elem.value.s
+    else:
+        return None
+
+
 def _view_source_node(env, text, state):
     # pretend we're using viewcode fully,
     # install the context it looks for
-    if not hasattr(env, '_viewcode_modules'):
+    if not hasattr(env, "_viewcode_modules"):
         env._viewcode_modules = {}
 
     modname = text
@@ -66,7 +80,8 @@ def _view_source_node(env, text, state):
         else:
             warnings.warn(
                 "Could not get base module for relative module: %s; "
-                "not generating node" % modname)
+                "not generating node" % modname
+            )
             return None
 
     urito = env.app.builder.get_relative_uri
@@ -75,16 +90,19 @@ def _view_source_node(env, text, state):
     # which we really don't want to have required so load the
     # module by file, not import (though we are importing)
     # the top level module here...
-    pathname = None
+    pathname = module_docstring = None
 
     for tok in modname.split("."):
         try:
             file_, pathname, desc = imp.find_module(
-                tok, [pathname] if pathname else None)
+                tok, [pathname] if pathname else None
+            )
         except ImportError as ie:
             raise ImportError("Error trying to import %s: %s" % (modname, ie))
         else:
             if file_:
+                if state:
+                    module_docstring = _get_module_docstring(file_)
                 file_.close()
 
     # unlike viewcode which silently traps exceptions,
@@ -99,21 +117,7 @@ def _view_source_node(env, text, state):
     else:
         code = analyzer.code
 
-    if state is not None:
-        docstring = _find_mod_docstring(analyzer)
-        if docstring:
-            # get rid of "foo.py" at the top
-            docstring = re.sub(r"^[a-zA-Z_0-9]+\.py", "", docstring)
-
-            # strip
-            docstring = docstring.strip()
-
-            # yank only first paragraph
-            docstring = docstring.split("\n\n")[0].strip()
-    else:
-        docstring = None
-
-    pagename = '_modules/' + modname.replace('.', '/')
+    pagename = "_modules/" + modname.replace(".", "/")
     try:
         refuri = urito(env.docname, pagename)
     except NoUri:
@@ -127,61 +131,47 @@ def _view_source_node(env, text, state):
         entry = code, analyzer.tags, {}
     env._viewcode_modules[modname] = entry
 
-    if docstring:
-        # embed the ref with the doc text so that it isn't
-        # a separate paragraph
-        if refuri:
-            docstring = "`%s <%s>`_ - %s" % (text, refuri, docstring)
-        else:
-            docstring = "``%s`` - %s" % (text, docstring)
-        para = nodes.paragraph('', '')
-        state.nested_parse(StringList([docstring]), 0, para)
-        return_node = para
+    if refuri:
+        refnode = nodes.reference(
+            "", "", nodes.Text(text, text), refuri=urito(env.docname, pagename)
+        )
     else:
-        if refuri:
-            refnode = nodes.reference(
-                '', '',
-                nodes.Text(text, text),
-                refuri=urito(env.docname, pagename)
-            )
-        else:
-            refnode = nodes.Text(text, text)
+        refnode = nodes.Text(text, text)
 
-        if state:
-            return_node = nodes.paragraph('', '', refnode)
-        else:
-            return_node = refnode
+    # get the first line of the module docstring
+    if module_docstring and state:
+        firstline = module_docstring.lstrip().split("\n\n")[0]
+        if 30 < len(firstline) < 450:  # opinionated
+            description_node = nodes.paragraph("", "")
+            # parse the content of the first line of the module
+            state.nested_parse(
+                docutils.statemachine.StringList([firstline]),
+                0,
+                description_node,
+            )
+            stuff_we_want = description_node.children[0].children
+            refnode = nodes.paragraph(
+                "", "", refnode, nodes.Text(" - ", " - "), *stuff_we_want
+            )
+
+    if state:
+        return_node = nodes.paragraph("", "", refnode)
+    else:
+        return_node = refnode
 
     return return_node
 
 
-def _find_mod_docstring(analyzer):
-    """attempt to locate the module-level docstring.
-
-    Note that sphinx autodoc just uses ``__doc__``.  But we don't want
-    to import the module, so we need to parse for it.
-
-    """
-    analyzer.tokenize()
-    for type_, parsed_line, start_pos, end_pos, raw_line in analyzer.tokens:
-        if type_ == token.COMMENT:
-            continue
-        elif type_ == token.STRING:
-            return eval(parsed_line)
-        else:
-            return None
-
-
 def _parse_content(content):
     d = {}
-    d['text'] = []
+    d["text"] = []
     idx = 0
     for line in content:
         idx += 1
-        m = re.match(r' *\:(.+?)\:(?: +(.+))?', line)
+        m = re.match(r" *\:(.+?)\:(?: +(.+))?", line)
         if m:
             attrname, value = m.group(1, 2)
-            d[attrname] = value or ''
+            d[attrname] = value or ""
         else:
             break
     d["text"] = content[idx:]
@@ -204,19 +194,21 @@ class AutoSourceDirective(Directive):
         sourcefile = self.state.document.current_source.split(os.pathsep)[0]
         dir_ = os.path.dirname(sourcefile)
         files = [
-            f for f in os.listdir(dir_) if f.endswith(".py")
-            and f != "__init__.py"
+            f
+            for f in os.listdir(dir_)
+            if f.endswith(".py") and f != "__init__.py"
         ]
 
         if "files" in content:
             # ordered listing of files to include
             files = [
-                fname for fname in _comma_list(content["files"])
-                if fname in set(files)]
+                fname
+                for fname in _comma_list(content["files"])
+                if fname in set(files)
+            ]
 
         node = nodes.paragraph(
-            '', '',
-            nodes.Text("Listing of files:", "Listing of files:")
+            "", "", nodes.Text("Listing of files:", "Listing of files:")
         )
 
         bullets = nodes.bullet_list()
@@ -227,10 +219,7 @@ class AutoSourceDirective(Directive):
 
             link = _view_source_node(env, modname, self.state)
             if link is not None:
-                list_node = nodes.list_item(
-                    '',
-                    link
-                )
+                list_node = nodes.list_item("", link)
                 bullets += list_node
 
         node += bullets
@@ -239,9 +228,9 @@ class AutoSourceDirective(Directive):
 
 
 def setup(app):
-    app.add_role('viewsource', view_source)
+    app.add_role("viewsource", view_source)
 
-    app.add_directive('autosource', AutoSourceDirective)
+    app.add_directive("autosource", AutoSourceDirective)
 
     # from sphinx.ext.viewcode
-    app.connect('html-collect-pages', collect_pages)
+    app.connect("html-collect-pages", collect_pages)
